
(defgroup twm-style "TWM style")

(defcustom twm:title-color (get-color "mediumvioletred")
  "Frame color"
  :group twm-style
  :type color
  :after-set after-setting-frame-option)

(defcustom twm:frame-color (get-color "cadetblue")
  "Frame color"
  :group twm-style
  :type color
  :after-set after-setting-frame-option)

(let*
    ((stipple-i (make-image "stipple.png"))
     (trans-i (make-image "trans.png"))
     (h-stipple-i (make-image "h_stipple.png"))
     (v-stipple-i (make-image "v_stipple.png"))
     (button-i (list (make-image "button.png") nil
		     nil (make-image "button-c.png")))

     (h-stipple (lambda () (list h-stipple-i twm:frame-color)))
     (v-stipple (lambda () (list v-stipple-i twm:frame-color)))

     (hashed-offset (lambda (w)
		      (+ (text-width (window-name w)) 24)))

     (update (lambda (w)
	       (if (eq (window-get w 'current-frame-style) 'twm)
		   (rebuild-frame w))))

     (frame `(((foreground . "white")
	       (background . ,(lambda () twm:title-color))
	       (text . ,window-name)
	       (x-justify . 20)
	       (y-justify . center)
	       (left-edge . 0)
	       (right-edge . 0)
	       (top-edge . -20)
	       (height . 18)
	       (class . title))
	      ((background . ,(lambda () twm:title-color))
	       (foreground . ,button-i)
	       (left-edge . 2)
	       (top-edge . -18)
	       (width . 14)
	       (height . 14)
	       (class . menu-button))
	      ((background . ,(lambda () (list twm:title-color stipple-i)))
	       (right-edge . 2)
	       (left-edge . ,hashed-offset)
	       (top-edge . -18)
	       (height . 14)
	       (class . title))
	      ((background . ,h-stipple)
	       (left-edge . 0)
	       (right-edge . 0)
	       (top-edge . -22)
	       (height . 2)
	       (class . top-border))
	      ((background . ,h-stipple)
	       (left-edge . 0)
	       (right-edge . 0)
	       (top-edge . -2)
	       (height . 2))
	      ((background . ,h-stipple)
	       (left-edge . 0)
	       (right-edge . 0)
	       (bottom-edge . -2)
	       (height . 2)
	       (class . bottom-border))
	      ((background . ,v-stipple)
	       (left-edge . -2)
	       (width . 2)
	       (top-edge . -22)
	       (bottom-edge . -2)
	       (class . left-border))
	      ((background . ,v-stipple)
	       (right-edge . -2)
	       (width . 2)
	       (top-edge . -22)
	       (bottom-edge . -2)
	       (class . right-border))))

     (shaped-frame `(((foreground . "white")
		      (background . ,(lambda () twm:title-color))
		      (text . ,window-name)
		      (x-justify . 20)
		      (y-justify . center)
		      (left-edge . 0)
		      (right-edge . 0)
		      (top-edge . -20)
		      (height . 18)
		      (class . title))
		     ((background . ,(lambda () twm:title-color))
		      (foreground . ,button-i)
		      (left-edge . 2)
		      (top-edge . -18)
		      (width . 14)
		      (height . 14)
		      (class . menu-button))
		     ((background . ,(lambda ()
				       (list twm:title-color stipple-i)))
		      (right-edge . 2)
		      (left-edge . ,hashed-offset)
		      (top-edge . -18)
		      (height . 14)
		      (class . title))
		     ((background . ,h-stipple)
		      (left-edge . 0)
		      (right-edge . 0)
		      (top-edge . -22)
		      (height . 2)
		      (class . top-border))
		     ((background . ,h-stipple)
		      (left-edge . 0)
		      (right-edge . 0)
		      (top-edge . -2)
		      (height . 2))
		     ((background . ,v-stipple)
		      (left-edge . -2)
		      (width . 2)
		      (top-edge . -22)
		      (height . 22)
		      (class . left-border))
		     ((background . ,v-stipple)
		      (right-edge . -2)
		      (width . 2)
		      (top-edge . -22)
		      (height . 22)
		      (class . right-border))))

     (transient-frame `(((background . ,h-stipple)
			 (left-edge . 0)
			 (right-edge . 0)
			 (top-edge . -2)
			 (height . 2)
			 (class . title))
			((background . ,h-stipple)
			 (left-edge . 0)
			 (right-edge . 0)
			 (bottom-edge . -2)
			 (height . 2)
			 (class . bottom-border))
			((background . ,v-stipple)
			 (left-edge . -2)
			 (width . 2)
			 (top-edge . -2)
			 (bottom-edge . -2)
			 (class . left-border))
			((background . ,v-stipple)
			 (right-edge . -2)
			 (width . 2)
			 (top-edge . -2)
			 (bottom-edge . -2)
			 (class . right-border))))

     (shaped-transient-frame `(((background . ,h-stipple)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -2)
				(height . 2)
				(class . title)))))

  (mapc (lambda (i)
	  (image-put i 'tiled t))
	(list stipple-i trans-i h-stipple-i v-stipple-i))

  (add-frame-style 'twm
		   (lambda (w type)
		     (cond ((eq type 'unframed)
			    nil-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaped-transient)
			    shaped-transient-frame)
			   ((eq type 'shaped)
			    shaped-frame)
			   (t
			    frame))))

  (call-after-property-changed 'WM_NAME update))
